/***************************************************************************
 *   Copyright (C) 2008 by Manuel Schrape                                  *
 *   manuel.schrape@gmx.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <QtGui>


#include <QLineEdit>
#include <QString>

#include "mktool.h"

MKTool::MKTool()
{
    setupUi(this);

    pb_Decode->hide();
    le_In->hide();
    le_Out->hide();

    read_Settings();

    setWindowTitle("QMK-Groundstation v0.0.5");

    serialPort=serialPort = new ManageSerialPort;
    connect(serialPort, SIGNAL(newDataReceived(const QByteArray &)), this, SLOT(slot_newDataReceived(const QByteArray &)));

    connect(pb_Decode,  SIGNAL(clicked()), this, SLOT(Decode64()));
    connect(pb_Open,  SIGNAL(clicked()), this, SLOT(slot_OpenPort()));
    connect(pb_Quit,  SIGNAL(clicked()), this, SLOT(slot_Quit()));
}

void MKTool::read_Settings()
{
    QSettings Setting("KeyOz-Net", "QMK-Groundstation");

    Setting.beginGroup("Port");
        le_Port->setText(Setting.value("TTY", QString("/dev/ttyUSB0")).toString());
    Setting.endGroup();

    Setting.beginGroup("GUI");
        resize(Setting.value("Size", QSize(700, 300)).toSize());
        move(Setting.value("Point",QPoint(1,1)).toPoint());
        if (Setting.value("IsMax",false).toBool())
            showMaximized();
    Setting.endGroup();

    Setting.beginGroup("AnalogWerte");
        lb_A_0->setText(Setting.value("Analog_0", QString("Integral Nick")).toString());
        lb_A_1->setText(Setting.value("Analog_1", QString("Integral Roll")).toString());
        lb_A_2->setText(Setting.value("Analog_2", QString("ACC Nick")).toString());
        lb_A_3->setText(Setting.value("Analog_3", QString("ACC Roll")).toString());
        lb_A_4->setText(Setting.value("Analog_4", QString("Gyro Gier")).toString());
        lb_A_5->setText(Setting.value("Analog_5", QString("Hoehen-Wert")).toString());
        lb_A_6->setText(Setting.value("Analog_6", QString("ACC Z")).toString());
        lb_A_7->setText(Setting.value("Analog_7", QString("GAS")).toString());
        lb_A_8->setText(Setting.value("Analog_8", QString("Kompass-Value")).toString());
        lb_A_9->setText(Setting.value("Analog_9", QString("Spannung")).toString());
        lb_A_10->setText(Setting.value("Analog_10", QString("Empfang")).toString());
        lb_A_11->setText(Setting.value("Analog_11", QString("Ersatzkompass")).toString());
        lb_A_12->setText(Setting.value("Analog_12", QString("Motor Vorne")).toString());
        lb_A_13->setText(Setting.value("Analog_13", QString("Motor Hinten")).toString());
        lb_A_14->setText(Setting.value("Analog_14", QString("Motor Links")).toString());
        lb_A_15->setText(Setting.value("Analog_15", QString("Motor Rechts")).toString());
        lb_A_17->setText(Setting.value("Analog_17", QString("Distance")).toString());
        lb_A_18->setText(Setting.value("Analog_18", QString("OSD-Bar")).toString());
        lb_A_19->setText(Setting.value("Analog_19", QString("MK3Mag")).toString());
        lb_A_20->setText(Setting.value("Analog_20", QString("Servo")).toString());
        lb_A_21->setText(Setting.value("Analog_21", QString("Nick")).toString());
        lb_A_22->setText(Setting.value("Analog_22", QString("Roll")).toString());
        lb_A_30->setText(Setting.value("Analog_30", QString("GPS Nick")).toString());
        lb_A_31->setText(Setting.value("Analog_31", QString("GPS Roll")).toString());
    Setting.endGroup();
}

void MKTool::Decode64()
{
    QString OUT = "";

    char *RxdBuffer;

    QString DecodeString = le_In->text();
    DecodeString.remove(0,3);

    RxdBuffer = DecodeString.toLatin1().data();

    unsigned char a,b,c,d;
    unsigned char ptr = 0;
    unsigned char x,y,z;
    int ptrOut[100];

    

    int len = DecodeString.length();
    int ptrIn = 0;
    int max = len;
    int DecLen = 0;

    while(len != 0)
    {
        a = RxdBuffer[ptrIn++] - '=';
        b = RxdBuffer[ptrIn++] - '=';
        c = RxdBuffer[ptrIn++] - '=';
        d = RxdBuffer[ptrIn++] - '=';

        if(ptrIn > max - 2) break; // nicht mehr Daten verarbeiten, als empfangen wurden

        x = (a << 2) | (b >> 4);
        y = ((b & 0x0f) << 4) | (c >> 2);
        z = ((c & 0x03) << 6) | d;

        if(len--) ptrOut[ptr++] = x; else break;
        if(len--) ptrOut[ptr++] = y; else break;
        if(len--) ptrOut[ptr++] = z; else break;
    }

    for (int a=0; a<ptr; a++)
    {
        int b1, b2, b3;

        b1 = ptrOut[a++];
        b2 = ptrOut[a];

        b3 = (b2 << 8) | b1;

        if (b3 > 32767)
            b3 = b3 - 65536;

        DecodeData[DecLen] = b3;
        DecLen++;
        OUT = OUT + QString(",%1").arg(b3);
    }

    le_Out->setText(OUT);
}

void MKTool::show_Data()
{
    le_A_0->setText(QString("%1").arg(AnalogData[0]));
    le_A_1->setText(QString("%1").arg(AnalogData[1]));
    le_A_2->setText(QString("%1").arg(AnalogData[2]));
    le_A_3->setText(QString("%1").arg(AnalogData[3]));
    le_A_4->setText(QString("%1").arg(AnalogData[4]));
    le_A_5->setText(QString("%1").arg(AnalogData[5]));
    le_A_6->setText(QString("%1").arg(AnalogData[6]));
    le_A_7->setText(QString("%1").arg(AnalogData[7]));
    le_A_8->setText(QString("%1").arg(AnalogData[8]));
    le_A_9->setText(QString("%1").arg(AnalogData[9]));
    le_A_10->setText(QString("%1").arg(AnalogData[10]));
    le_A_11->setText(QString("%1").arg(AnalogData[11]));
    le_A_12->setText(QString("%1").arg(AnalogData[12]));
    le_A_13->setText(QString("%1").arg(AnalogData[13]));
    le_A_14->setText(QString("%1").arg(AnalogData[14]));
    le_A_15->setText(QString("%1").arg(AnalogData[15]));
    le_A_16->setText(QString("%1").arg(AnalogData[16]));
    le_A_17->setText(QString("%1").arg(AnalogData[17]));
    le_A_18->setText(QString("%1").arg(AnalogData[18]));
    le_A_19->setText(QString("%1").arg(AnalogData[19]));
    le_A_20->setText(QString("%1").arg(AnalogData[20]));
    le_A_21->setText(QString("%1").arg(AnalogData[21]));
    le_A_22->setText(QString("%1").arg(AnalogData[22]));
    le_A_23->setText(QString("%1").arg(AnalogData[23]));
    le_A_24->setText(QString("%1").arg(AnalogData[24]));
    le_A_25->setText(QString("%1").arg(AnalogData[25]));
    le_A_26->setText(QString("%1").arg(AnalogData[26]));
    le_A_27->setText(QString("%1").arg(AnalogData[27]));
    le_A_28->setText(QString("%1").arg(AnalogData[28]));
    le_A_29->setText(QString("%1").arg(AnalogData[29]));
    le_A_30->setText(QString("%1").arg(AnalogData[30]));
    le_A_31->setText(QString("%1").arg(AnalogData[31]));
}

void MKTool::rx_NewData(QString RX_Data)
{
    char *RX = RX_Data.toLatin1().data();

    switch(RX[2])
    {
        case 'D' :
                    le_In->setText(RX_Data);
                    Decode64();
                    for (int i=0; i<32; i++)
                    {
                        AnalogData[i] = DecodeData[i+1];
                    }
                    show_Data();
                break;
    }
}

void MKTool::slot_OpenPort()
{
    if (serialPort->isOpen())
    {
        serialPort->close();
        pb_Open->setText("Verbinden");
    }
    else
    {
        serialPort->setPort(le_Port->text()); //Port

        serialPort->setBaudRate(BAUD57600); //BaudRate

        serialPort->setDataBits(DATA_8); //DataBits

        serialPort->setParity(PAR_NONE); //Parity

        serialPort->setStopBits(STOP_1); //StopBits

        serialPort->setFlowControl(FLOW_OFF); //FlowControl

        serialPort->setTimeout(0, 10);
        serialPort->enableSending();
        serialPort->enableReceiving();

        serialPort->open();
        serialPort->receiveData();

        pb_Open->setText("Trennen");
    }
}

bool MKTool::check_CRC(QString RXString)
{
    int CRC = 0;
    char *RX;

    int Length = RXString.length();

    RX = RXString.toLatin1().data();

    for(int i=0; i < Length-2; i++)
    {
            CRC+=RX[i];
    }

    CRC -= RX[1];

    CRC = CRC % 4096;

    if(RX[Length - 2] != ('=' + (CRC / 64)))
    {
        return false;
    }

    if(RX[Length - 1] != ('=' + CRC % 64))
    {
        return false;
    }

    return true;
}

void  MKTool::slot_newDataReceived(const QByteArray &dataReceived)
{
    allDataReceived += dataReceived;

    te_RX->moveCursor(QTextCursor::End,  QTextCursor::MoveAnchor);

    QTextStream in(dataReceived);

    QString decodedStr = in.readAll();

    const char *RXt;
    RXt = dataReceived.data();
    int a = 0;

    while (RXt[a] != '\0')
    {
        if (RXt[a] == '\r')
        {
            if (check_CRC(RXS))
            {
                te_RX->insertPlainText(RXS + '\r');
                rx_NewData(RXS);
            }

            RXS = QString("");
        }
        else
        {
            RXS = RXS + QString(RXt[a]);
        }
            a++;
    }
}

void MKTool::write_Settings()
{
    QSettings Setting("KeyOz-Net", "QMK-Groundstation");

    Setting.beginGroup("Port");
        Setting.setValue("TTY", le_Port->text());
    Setting.endGroup();

    Setting.beginGroup("GUI");
        Setting.setValue("IsMax", isMaximized());
        Setting.setValue("Size",  size());
        Setting.setValue("Point", pos());
    Setting.endGroup();

    Setting.beginGroup("AnalogWerte");
        Setting.setValue("Analog_0", lb_A_0->text());
        Setting.setValue("Analog_1", lb_A_1->text());
        Setting.setValue("Analog_2", lb_A_2->text());
        Setting.setValue("Analog_3", lb_A_3->text());
        Setting.setValue("Analog_4", lb_A_4->text());
        Setting.setValue("Analog_5", lb_A_5->text());
        Setting.setValue("Analog_6", lb_A_6->text());
        Setting.setValue("Analog_7", lb_A_7->text());
        Setting.setValue("Analog_8", lb_A_8->text());
        Setting.setValue("Analog_9", lb_A_9->text());
        Setting.setValue("Analog_10", lb_A_10->text());
        Setting.setValue("Analog_11", lb_A_11->text());
        Setting.setValue("Analog_12", lb_A_12->text());
        Setting.setValue("Analog_13", lb_A_13->text());
        Setting.setValue("Analog_14", lb_A_14->text());
        Setting.setValue("Analog_15", lb_A_15->text());
        Setting.setValue("Analog_16", lb_A_16->text());
        Setting.setValue("Analog_17", lb_A_17->text());
        Setting.setValue("Analog_18", lb_A_18->text());
        Setting.setValue("Analog_19", lb_A_19->text());
        Setting.setValue("Analog_20", lb_A_20->text());
        Setting.setValue("Analog_21", lb_A_21->text());
        Setting.setValue("Analog_22", lb_A_22->text());
        Setting.setValue("Analog_23", lb_A_23->text());
        Setting.setValue("Analog_24", lb_A_24->text());
        Setting.setValue("Analog_25", lb_A_25->text());
        Setting.setValue("Analog_26", lb_A_26->text());
        Setting.setValue("Analog_27", lb_A_27->text());
        Setting.setValue("Analog_28", lb_A_28->text());
        Setting.setValue("Analog_29", lb_A_29->text());
        Setting.setValue("Analog_30", lb_A_30->text());
        Setting.setValue("Analog_31", lb_A_31->text());
    Setting.endGroup();
}

void MKTool::slot_Quit()
{
    if (serialPort->isOpen())
    {
        serialPort->close();
        pb_Open->setText("Verbinden");
    }

    write_Settings();
    close();
}

MKTool::~MKTool()
{
    write_Settings();
}

